C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Budall version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.0 BUDALL 23JUNE1997
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Allocates space within the common array for cell-by-cell budget terms
C      polygon coverages and attributes.  These cell-by-cell budget terms
C      are output by the different packages of the U.S. Geological Survey 
C      Modular Three-Dimensional Finite-Differences Ground-Water Flow Model
C      MODFLOW or MODFLOWARC (McDonald and Harbaugh, 1988; Orzol and McGrath,
C      1992). These packages are:
C                         (1) River [RIV1];
C                         (2) Evapotranspiration [EVT1];
C                         (3) Well [WEL1];
C                         (4) Recharge [RCH1];
C                         (5) Drain [DRN1];
C                         (6) General-Head boundary [GHB1];
C                         (7) Block-Centered (storage only) [BCF1 or BCF2].
C      To generate polygon coverages from the cell-by-cell flow terms of
C      Block-Centered Package (constant_heads, flow_front_face, 
C      flow_lower_face, and flow_right_face) use the Modtools module
C      Cell_By_Cell [CBC1].
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT DATA FILE
C     LENX    : MAXIMUM DIMENSION OF THE COMMON BLOCK ARRAY
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C
C   Inputs/Outputs:
C
C     ISUM    : INDEX NUMBER TO COMMON BLOCK ARRAY FOR ALLOCATION
C
C    Outputs:
C
C     LCBUDT  : STARTING ARRAY INDEX FOR CELL_BY_CELL BUDGET TERMS
C
C     Program:
C
C     ISUM1   : INDEX NUMBER TO LAST COMMON BLOCK ARRAY FOR ALLOCATION
C
C-------------------------------------------------------------------------
      SUBROUTINE BUDALL (
     I                   OUTUNT,
     I                   LENX,
     I                   NLAY,NROW,NCOL,
     B                   ISUM,
     O                   LCBUDT,
     E                   *)
C-----------------------------------------------------------------------
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      INTEGER
     &        OUTUNT,
     &        LENX,
     &        NLAY,NROW,NCOL,
     &        ISUM,
     &        LCBUDT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          CELL-BY-CELL BUDGET TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.0 23JUNE1997 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                 ALLOCATION MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1----Calculate storage parameter NRCL
C
      NRCL=NROW*NCOL*NLAY
C
C2----Allocate space for arrays
C
      LCBUDT=ISUM
      ISUM=ISUM+NRCL
      ISUM1=ISUM-1
C
C3----Print Modtools operation parameters
C
      WRITE(OUTUNT,'(//,1H1,38X,
     &             ''COMMON BLOCK INDEX LISTING'',
     &             /10X,85(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &             ''MAXIMUM NUMBER OF CELL BUDGET TERMS            '',
     &             10X,I15,'' NRCL'')')
     &          NRCL
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR BUDGET TERMS          '',
     &             10X,I15,'' LCBUDT'')')
     &          LCBUDT
      WRITE(OUTUNT,'(10X,85(''-''))')
C
      ISP=ISUM1-ISP
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS IN COMMON BLOCK ARRAY USED '',
     &             ''FOR CELL-BY-CELL BUDGET TERMS PACKAGE'')')
     &          ISP
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS OF COMMON BLOCK ARRAY USED OUT OF '',
     &             I8)')
     &          ISUM1,LENX
C
C4----Check amount of space used
C
      IF(ISUM1.GT.LENX) THEN
         WRITE(OUTUNT,'(//,
     &                ''    ***CELL ARRAY MUST BE DIMENSIONED '',
     &                ''LARGER*** ISUM '',I10,'' > LENX '',I10)')
     &             ISUM1,
     &             LENX
         WRITE(OUTUNT,'(//,
     &             '' Abnormal termination of Budall_Subroutine'')')
         WRITE (*,'(//,'' Cell array must be dimensioned larger ISUM '',
     &              I10,'' > LENX '',I10)')
     &           ISUM1,
     &           LENX
         WRITE(*,'(//,
     &             '' Abnormal termination of Budall_Subroutine'')')
         RETURN 1
      ENDIF
C
C5----Return
C
      RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Budrp1 version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.0 BUDRP1 30AUGUST1999
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Reads an user defined files for cell_by_cell flow terms.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTFIL  : OUTPUT FILE NAME DOCUMENTING HEDRP1 OPERATIONS.
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT DATA FILE
C     BUDFIL  : FILE CONTAINING CELL_BY_CELL FLOW TERMS FILE
C     BUDUNT  : FORTRAN UNIT NUMBER CELL_BY_CELL FLOW TERMS FILE
C     CTLFIL  : FILE CONTAINING THE CONTROL LINES
C     CTLUNT  : FORTRAN UNIT NUMBER FOR U3DREL CONTROL LINES
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C
C    Outputs:
C
C     BUDARY  : ARRAY FOR CELL_BY_CELL FLOW TERMS
C
C     Program:
C
C     ANAME   : CHARACTER ARRAY (6,8) CONTAINING ARRAY NAMES
C     K       : LOOP INDEX
C     KK      : PASSED LAYER NUMBER
C     LOC     : STARTING INDEX FOR ARRAY
C
C-------------------------------------------------------------------------
      SUBROUTINE BUDRP1 (
     I                   OUTFIL,OUTUNT,
     I                   BUDFIL,BUDUNT,
     I                   CTLFIL,CTLUNT,
     I                   MAXPMR,
     I                   NLAY,NROW,NCOL,
     I                   COVPMR,
     O                   BUDARY,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION COVPMR(MAXPMR),
     &          BUDARY(NCOL*NROW*NLAY)
C
      CHARACTER*(*) OUTFIL,BUDFIL,CTLFIL
C
      INTEGER OUTUNT,BUDUNT,CTLUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*16 BUDTXT(12),TEXT
      CHARACTER*4 ANAME(6,1)
C
      INTEGER ICOL,IROW,ILAY,LAYER,KK,BCFFLG,CBCUNT
C
      DATA BUDTXT(1)  /'         STORAGE'/
      DATA BUDTXT(2)  /'   RIVER LEAKAGE'/
      DATA BUDTXT(3)  /'        RECHARGE'/
      DATA BUDTXT(4)  /'           WELLS'/
      DATA BUDTXT(5)  /'          DRAINS'/
      DATA BUDTXT(6)  /' HEAD DEP BOUNDS'/
      DATA BUDTXT(7)  /'              ET'/
      DATA BUDTXT(8)  /'  STREAM LEAKAGE'/
      DATA BUDTXT(9)  /'  STREAM FLOWOUT'/
      DATA BUDTXT(10) /'INTERBED STORAGE'/
C
      DATA ANAME(1,1),ANAME(2,1),ANAME(3,1),ANAME(4,1),ANAME(5,1),
     1     ANAME(6,1) /'CELL','_BY_','CELL',' BUD','GET ','TERM'/
C
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                    Cell_By_Cell Flow Array
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          CELL-BY-CELL BUDGET TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.0 30AUGUST1999 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                READ/PREPARE MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
CL----Set local variables
C
      MODFLG=INT(COVPMR(1))
      MODFMT=MODFLG-3
      MODPER=INT(COVPMR(13))
      MODSTP=INT(COVPMR(14))
      NPER=INT(COVPMR(17))
C
      IF(NPER.EQ.0) THEN
         MODPER=1
         MODSTP=1
         WRITE(*,'(
     &          ''      .Reading steady-state cell_by_cell terms'')')
         WRITE(OUTUNT,'(
     &          ''      .Reading steady-state cell_by_cell terms'')')
      ELSE
         WRITE(*,'(
     &          ''      .Reading transient cell_by_cell terms'')')
         WRITE(OUTUNT,'(
     &          ''      .Reading transient cell_by_cell terms'')')
      ENDIF
C
      IF(MODFMT.EQ.1) THEN
         WRITE(*,'(
     &          ''      .Reading Cell-By-Cell Storage Terms from '',
     &          ''Block-Centered Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Reading Cell-By-Cell Storage Terms from '',
     &          ''Block-Centered Package '')')
      ELSE IF(MODFMT.EQ.2) THEN
         WRITE(*,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''River Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''River Package '')')
      ELSE IF(MODFMT.EQ.3) THEN
         WRITE(*,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''Recharge Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''Recharge Package '')')
      ELSE IF(MODFMT.EQ.4) THEN
         WRITE(*,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''Well Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''Well Package '')')
      ELSE IF(MODFMT.EQ.5) THEN
         WRITE(*,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''Drain Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''Drain Package '')')
      ELSE IF(MODFMT.EQ.6) THEN
         WRITE(*,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''General Head Boundary Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''General Head Boundary Package '')')
      ELSE IF(MODFMT.EQ.7) THEN
         WRITE(*,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''Evapotranspiration Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''Evapotranspiration Package '')')
      ELSE IF(MODFMT.EQ.8) THEN
         WRITE(*,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''Streamflow-Routing Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Reading Cell-By-Cell Flow Terms from '',
     &          ''Streamflow-Routing Package '')')
      ELSE IF(MODFMT.EQ.9) THEN
         WRITE(*,'(
     &          ''      .Reading Cell-By-Cell Streamflow Terms  '',
     &          ''from Streamflow-Routing Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Reading Cell-By-Cell Streamflow Terms '',
     &          ''from Streamflow-Routing Package '')')
      ELSE IF(MODFMT.EQ.10) THEN
         WRITE(*,'(
     &          ''      .Reading Interbed-Storage Terms from '',
     &          ''Aquifer-System Compaction Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Reading Interbed-Storage Terms from '',
     &          ''Aquifer-System Compaction Package '')')
      ELSE
         WRITE(*,'(
     &          ''      .Reading Miscellaneous Gridded Terms '')')
         WRITE(OUTUNT,'(
     &          ''      .Reading Miscellaneous Gridded Terms '')')
      ENDIF
C
C1---Binary formatted cell-by-cell head terms [MODFLOW data]
C
      IF(BUDUNT.LT.0 .AND. MODFMT.LE.9) THEN
         WRITE(*,'(
     &             ''      .Reading unformatted cell_by_cell '',
     &             ''flow terms '')')
         WRITE(OUTUNT,'(
     &             ''      .Reading unformatted cell_by_cell '',
     &             ''flow terms '')')
C
C1A---Open unformatted file BUDFIL
C
         NSTAT=1
         ITALK=1
         CBCUNT=ABS(BUDUNT)
         CALL OPEN_FILE (BUDUNT,BUDFIL,NSTAT,ITALK,*9999)
         WRITE(*,'(''      ..recorded in file '',A)')
     &             BUDFIL (1:INDEX(BUDFIL,' ')-1)
         WRITE(*,'(''      ..on unit '',I10)')
     &                                                CBCUNT
         WRITE(OUTUNT,'(''      ..recorded in file '',A)')
     &          BUDFIL (1:INDEX(BUDFIL,' ')-1)
         WRITE(OUTUNT,'(''      ..on unit '',I10)')
     &                                                CBCUNT
C
C1B---Loop to read from unformatted file BUDFIL
C
         BUDFLG=0
         WRITE(*,'(
     &             ''      ..Loading cell_by_cell flow terms'')')
         WRITE(OUTUNT,'(
     &             ''      ..Loading cell_by_cell flow terms '')')
C
  102    READ(CBCUNT,ERR=9990,END=104) 
     &                                 KSTP,
     &                                 KPER,
     &                                 TEXT,
     &                                 ICOL,
     &                                 IROW,
     &                                 ILAY
       IF(INDEX(TEXT,BUDTXT(MODFMT)).NE.0 .AND.
     &    KPER.EQ.MODPER .AND. KSTP.EQ.MODSTP) THEN
          BUDFLG=BUDFLG+1
          READ(CBCUNT,ERR=9991,END=9992) BUDARY
          WRITE(OUTUNT,('(1X,///11X,A,'' FOR ALL LAYERS '',/,
     &                       1X,'' READING BINARY ON UNIT '',I4)'))
     &             TEXT,CBCUNT
          GO TO 104
       ELSE
          READ(CBCUNT,ERR=9991,END=9992) BUDARY
          GO TO 102
       ENDIF
C
  104  IF(BUDFLG.LE.0) THEN
          WRITE(*,'(
     &             ''      .Did not find cell_by_cell flow '',
     &             ''terms in file '',A)')
     &             BUDFIL (1:INDEX(BUDFIL,' ')-1)
          WRITE(OUTUNT,'(
     &             ''      .Did not find cell_by_cell flow '',
     &             ''terms in file '',A)')
     &             BUDFIL (1:INDEX(BUDFIL,' ')-1)
       ENDIF
C
C1C---Close BUDFIL file
C
         CALL CLOSE_FILE (BUDFIL,*9999)
C
C2---Binary formatted cell-by-cell head terms [Miscellaneous gridded data]
C
      ELSE IF(BUDUNT.LT.0 .AND. MODFMT.GT.9) THEN
         WRITE(*,'(
     &             ''      .Reading unformatted miscellaneous '',
     &             ''gridded data '')')
         WRITE(OUTUNT,'(
     &             ''      .Reading unformatted miscellaneous '',
     &             ''gridded data '')')
C
C2A---Open unformatted file BUDFIL
C
         NSTAT=1
         ITALK=1
         CBCUNT=ABS(BUDUNT)
         CALL OPEN_FILE (BUDUNT,BUDFIL,NSTAT,ITALK,*9999)
         WRITE(*,'(''      ..recorded in file '',A)')
     &             BUDFIL (1:INDEX(BUDFIL,' ')-1)
         WRITE(*,'(''      ..on unit '',I10)')
     &                                                CBCUNT
         WRITE(OUTUNT,'(''      ..recorded in file '',A)')
     &          BUDFIL (1:INDEX(BUDFIL,' ')-1)
         WRITE(OUTUNT,'(''      ..on unit '',I10)')
     &                                                CBCUNT
C
C2B---Loop to read from unformatted file BUDFIL
C
         BUDFLG=0
         WRITE(*,'(
     &             ''      ..Loading miscellaneous gridded data'')')
         WRITE(OUTUNT,'(
     &             ''      ..Loading miscellaneous gridded data'')')
C
 202     READ(CBCUNT,ERR=9990,END=204) 
     &                                 KSTP,
     &                                 KPER,
     &                                 TEXT,
     &                                 ICOL,
     &                                 IROW,
     &                                 ILAY
       IF(KPER.EQ.MODPER .AND. KSTP.EQ.MODSTP) THEN
          BUDFLG=BUDFLG+1
          READ(CBCUNT,ERR=9991,END=9992) BUDARY
          WRITE(OUTUNT,('(1X,///11X,A,'' FOR ALL LAYERS '',/,
     &                       1X,'' READING BINARY ON UNIT '',I4)'))
     &             TEXT,CBCUNT
          GO TO 204
       ELSE
          READ(CBCUNT,ERR=9991,END=9992) BUDARY
          GO TO 202
       ENDIF
C
 204   IF(BUDFLG.LE.0) THEN
          WRITE(*,'(
     &             ''      .Did not find miscellaneous gridded data '',
     &             ''terms in file '',A)')
     &             BUDFIL (1:INDEX(BUDFIL,' ')-1)
          WRITE(OUTUNT,'(
     &             ''      .Did not find miscellaneous gridded data '',
     &             ''terms in file '',A)')
     &             BUDFIL (1:INDEX(BUDFIL,' ')-1)
       ENDIF
C
C2C---Close BUDFIL file
C
         CALL CLOSE_FILE (BUDFIL,*9999)
C
C3---Text formatted cell-by-cell flow terms [MODFLOW data]
C
      ELSE IF(BUDUNT.GT.0 .AND. MODFMT.LE.9) THEN
         WRITE(*,'(
     &             ''      .Reading text formatted cell_by_cell '',
     &             ''flow terms '')')
         WRITE(OUTUNT,'(
     &             ''      .Reading text formatted cell_by_cell '',
     &             ''flow terms '')')
C
C3A----Open control file CTLFIL and text formatted file BUDFIL
C
       IF(CTLUNT.GT.0) THEN
          NSTAT=1
          ITALK=1
          CALL OPEN_FILE (CTLUNT,CTLFIL,NSTAT,ITALK,*9999)
          CALL OPEN_FILE (BUDUNT,BUDFIL,NSTAT,ITALK,*9999)
          WRITE(*,'(
     &              ''      ..recorded in file '',A)')
     &              BUDFIL (1:INDEX(BUDFIL,' ')-1)
          WRITE(*,'(
     &              ''      ..on unit '',I10)')
     &                                                 BUDUNT
          WRITE(*,'(
     &              ''      .Using control lines '')')
          WRITE(*,'(
     &              ''      ..recorded in file '',A)')
     &              CTLFIL (1:INDEX(CTLFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              BUDFIL (1:INDEX(BUDFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..on unit '',I10)')
     &                                                 BUDUNT
          WRITE(OUTUNT,'(
     &              ''      .Using control lines '')')
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              CTLFIL (1:INDEX(CTLFIL,' ')-1)
       ELSE
          WRITE(*,'(
     &              ''      .Can not read text formatted '',
     &              ''cell_by_cell flow terms '')')
          WRITE(*,'(
     &              ''      ..recorded in file '',A)')
     &              BUDFIL (1:INDEX(BUDFIL,' ')-1)
          WRITE(*,'(
     &              ''      ..on unit '',I10)')
     &                                                 BUDUNT
          WRITE(*,'(
     &              ''      .Need file containing U3DREL control '',
     &              ''lines for format of cell-by-cell flow file'')') 
          WRITE(OUTUNT,'(
     &              ''      .Can not read text formatted '',
     &              ''cell_by_cell flow terms '')')
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              BUDFIL (1:INDEX(BUDFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..on unit '',I10)')
     &                                                 BUDUNT
          WRITE(OUTUNT,'(
     &              ''      .Need file containing U3DREL control '',
     &              ''lines for format of cell-by-cell flow file'')') 
          CALL CLOSE_FILE (BUDFIL,*9999)
          GO TO 9999
       ENDIF
C
C3B---Loop to read from text formatted file BUDFIL
C
         CALL U3DREL (BUDARY,ANAME(1,1),NROW,NCOL,NLAY,BUDUNT,OUTUNT)
C
C3C---Close text formatted file BUDFIL
C
         CALL CLOSE_FILE (BUDFIL,*9999)
         CALL CLOSE_FILE (CTLFIL,*9999)
C
C4---Text formatted cell-by-cell flow terms [MODFLOW data]
C
      ELSE IF(BUDUNT.GT.0 .AND. MODFMT.GT.9) THEN
         WRITE(*,'(
     &             ''      .Reading text formatted miscellaneous '',
     &             ''gridded data '')')
         WRITE(OUTUNT,'(
     &             ''      .Reading text formatted miscellaneous '',
     &             ''gridded data '')')
C
C4A----Open control file CTLFIL and text formatted file BUDFIL
C
       IF(CTLUNT.GT.0) THEN
          NSTAT=1
          ITALK=1
          CALL OPEN_FILE (CTLUNT,CTLFIL,NSTAT,ITALK,*9999)
          CALL OPEN_FILE (BUDUNT,BUDFIL,NSTAT,ITALK,*9999)
          WRITE(*,'(
     &              ''      ..recorded in file '',A)')
     &              BUDFIL (1:INDEX(BUDFIL,' ')-1)
          WRITE(*,'(
     &              ''      ..on unit '',I10)')
     &                                                 BUDUNT
          WRITE(*,'(
     &              ''      .Using control lines '')')
          WRITE(*,'(
     &              ''      ..recorded in file '',A)')
     &              CTLFIL (1:INDEX(CTLFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              BUDFIL (1:INDEX(BUDFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..on unit '',I10)')
     &                                                 BUDUNT
          WRITE(OUTUNT,'(
     &              ''      .Using control lines '')')
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              CTLFIL (1:INDEX(CTLFIL,' ')-1)
       ELSE
          WRITE(*,'(
     &              ''      .Can not read text formatted '',
     &              ''miscellaneous gridded data '')')
          WRITE(*,'(
     &              ''      ..recorded in file '',A)')
     &              BUDFIL (1:INDEX(BUDFIL,' ')-1)
          WRITE(*,'(
     &              ''      ..on unit '',I10)')
     &                                                 BUDUNT
          WRITE(*,'(
     &              ''      .Need file containing U3DREL control '',
     &              ''lines for format of miscellaneous data file'')') 
          WRITE(OUTUNT,'(
     &              ''      .Can not read text formatted '',
     &              ''miscellaneous data '')')
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              BUDFIL (1:INDEX(BUDFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..on unit '',I10)')
     &                                                 BUDUNT
          WRITE(OUTUNT,'(
     &              ''      .Need file containing U3DREL control '',
     &              ''lines for format of miscellaneous data file'')') 
          CALL CLOSE_FILE (BUDFIL,*9999)
          GO TO 9999
       ENDIF
C
C4B---Loop to read from text formatted file BUDFIL
C
         CALL U3DREL (BUDARY,ANAME(1,1),NROW,NCOL,NLAY,BUDUNT,OUTUNT)
C
C4C---Close text formatted file BUDFIL
C
         CALL CLOSE_FILE (BUDFIL,*9999)
         CALL CLOSE_FILE (CTLFIL,*9999)
C
C5---Error no file containing cell-by-cell flow terms
C
      ELSE
         WRITE(*,'(
     &             ''      .No file containing cell_by_cell '',
     &             ''flow terms specified in MODPATH name file'')')
         WRITE(OUTUNT,'(
     &             ''      .No file containing cell_by_cell '',
     &             ''flow terms specified in MODPATH name file'')')
         GO TO 9999
      ENDIF
        GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9990 WRITE(*,'(//,
     &          '' Unable to read header line for '',
     &          ''flow terms from file '',A)') 
     &          BUDFIL (1:INDEX(BUDFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read header line for '',
     &          ''flow terms from file '',A)') 
     &          BUDFIL (1:INDEX(BUDFIL,' ')-1)
      GO TO 9998
 9991 WRITE(*,'(//,
     &          '' Unable to read values for '',
     &          ''flow terms from file '',A)') 
     &          BUDFIL (1:INDEX(BUDFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read values for '',
     &          ''flow terms from file '',A)') 
     &          BUDFIL (1:INDEX(BUDFIL,' ')-1)
      GO TO 9998
 9992 WRITE(*,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''flow terms from file '',A)') 
     &          BUDFIL (1:INDEX(BUDFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''flow terms from file '',A)') 
     &          BUDFIL (1:INDEX(BUDFIL,' ')-1)
      GO TO 9998
 9998 CALL CLOSE_FILE (BUDFIL,*9999)
 9999 WRITE(*,
     &      '(//,'' Abnormal Termination of Budrp1_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal Termination of Budrp1_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Pthbud version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.0 PTHBUD 24SEPTEMBER1999
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Generates a Info file BUDFIL using the cell-by-cell 
C      budget terms that are output by the different packages of the 
C      U.S. Geological Survey Modular Three-Dimensional Finite-Differences
C      Ground-Water Flow Model MODFLOW or MODFLOWARC (McDonald and Harbaugh, 
C      1988; Orzol and McGrath, 1992). These packages are:
C                         (1) River [RIV1];
C                         (2) Evapotranspiration [EVT1];
C                         (3) Well [WEL1];
C                         (4) Recharge [RCH1];
C                         (5) Drain [DRN1];
C                         (6) General-Head boundary [GHB1];
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C      IVIEW
C           => 0 Mapview --> all layers
C                 One info files of cell-by-cell budget terms
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4+NLAY) LAYER_xx = CELL_BY_CELL BUDGET TERMS
C
C           => 1 Vertical row profile [appended to row coverage]
C                 Cell-by-cell budget terms attached to cell
C              => (n,1) CELL POLYGON NUMBER
C              => (n,2+NLAY) CELL_BY_CELL BUDGET TERMS
C
C           => 2 Vertical column profile
C                 Cell-by-cell budget terms attached to cell
C              => (n,1) CELL POLYGON NUMBER
C              => (n,2+NLAY) LAYER_xx = CELL_BY_CELL BUDGET TERMS
C
C           => 3 Vertical particle profile
C                 Cell-by-cell budget terms attached to cell
C              => (n,1) CELL POLYGON NUMBER
C              => (n,2+NLAY) LAYER_xx = CELL_BY_CELL BUDGET TERMS
C
C           => 4 Mapview --> user specified layer
C                 One info file of cell-by-cell budget terms
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) LAYER_xx = CELL_BY_CELL BUDGET TERMS FOR CHOSEN LAYER
C
C           => 5 Mapview --> user specified ibound value
C                 One info file of cell-by-cell budget terms
C                 (this ibound value can be used to differentiate model
C                  layers into geological units.)
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) CELL LAYER
C              => (n,5) CELL IBOUND VALUE
C              => (n,6) CELL_BY_CELL BUDGET TERMS 
C                       (these terms are computed as a summation of the 
C                         cell-by-cell budget terms for each layer which
C                         match the chosen ibound value.)
C           => 6 Mapview --> uppermost active cell
C                 One info file of cell-by-cell budget terms
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) CELL LAYER
C              => (n,5) CELL IBOUND VALUE
C              => (n,6) CELL_BY_CELL BUDGET TERMS
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C
C    Inputs:
C
C     BUDFIL  : COVERAGE ROOT NAME
C     COVPMR  : COVERAGE PARAMETERS ARRAY (MAXPMR)
C     DELX    : ARRAY FOR CELL WIDTHS IN ROW/COLUMN/PARTICLE DIRECTION (NDELX)
C     BUDARY  : ARRAY FOR LAYER/UNIT CELL_BY_CELL BUDGETS
C     IBOUND  : ARRAY FOR IBOUND ARRAY (NCOL,NROW,NLAY)
C     IOUTPT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED
C     IRWCL   : WHERE CROSS-SECTION TRANVERSE IS TAKEN
C     IVIEW   : FLAG (ORIENTATION OF COVERAGE)
C     LRWCL   : ARRAY INDICATING THE CELL'S ROW AND COLUMN (2,NDELX)
C     MAXATB  : MAXIMUM NUMBER OF ATTRIBUTES FOR EACH POLYGON
C                IN THE COVERAGE PAT FILE.
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS
C     MAXPTS  : MAXIMUM NUMBER OF VERTICES IN A SINGLE ARC
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     ZTOP    : ARRAY FOR LAYER/UNIT TOP ELEVATIONS
C     ZBOT    : ARRAY FOR LAYER/UNIT BOTTOM ELEVATIONS
C
C    Outputs:
C
C     PATLAB  : POLYGON ATTRIBUTE ARRAY (MAXPTS,MAXATB)
C             => (n,1) CELL POLYGON NUMBER
C             => (n,2) CELL COLUMN
C             => (n,3) CELL ROW
C             => (n,4) CELL LAYER
C             => (n,5) MODEL IBOUND VALUE
C             => (n,6) CELL TOP ELEVATION
C             => (n,7) CELL BOTTOM ELEVATION
C             => (n,8) to (n,20) Unused
C
C-----------------------------------------------------------------------
C
      SUBROUTINE PTHBUD (
     I                   OUTUNT,
     I                   BUDFIL,
     I                   MAXPMR,
     I                   NLAY,NROW,NCOL,
     I                   COVPMR,DELZCB,
     I                   LRWCL,
     I                   IBOUND,
     I                   BUDARY,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION 
     1          COVPMR(MAXPMR),
     1          DELZCB(NLAY),
     2          LRWCL(2,NCOL*NROW),
     3          IBOUND(NCOL,NROW,NLAY),
     3          BUDARY(NCOL,NROW,NLAY)
C
      CHARACTER*(*) BUDFIL
      INTEGER OUTUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      PARAMETER (MAXFIL=3,
     &           MAXITM=5)
C
      REAL BUDIBD
C
      CHARACTER*128 LABFIL,FMTSTR
      CHARACTER*80 FILES(MAXFIL),ADDFIL,INPFIL,TMPFIL
      CHARACTER*16 COVTYP,ITEM,ITEMS(MAXITM),ARCCMD,ACTION,
     &             LAYITM,STRITM,RELITM,JONTYP
      CHARACTER*10 CHRLAY,CHRWCL,LAYFMT,ITMFMT
      CHARACTER*6 MODTXT
      CHARACTER*1 TYPITM,TYPES(MAXITM)
C
      INTEGER DECIML,DECIMLS(MAXITM),IOUT(MAXFIL),ITMSTR,ITMFIN,
     &        OUTPUT,OUTPUTS(MAXITM),WIDTH,WIDTHS(MAXITM)
C
      INTEGER 
     &        IRWCL,IVIEW,
     &        IDPART,ICOLOR,TIMSTP,OLDSTP,
     &        STRCHR,STRCOL,STRROW,STRLAY,STRIBD,STRDEL,STRCLL,
     &        PRECOL,PREROW,PRELAY,PREIBD,
     &        FINCHR,FINCOL,FINROW,FINLAY,FINIBD,FINDEL,FINCLL,
     &        ROW,COLUMN,CELL,BADIBD,COVDEX,COL,ROWCOL,SEQNUM,
     &        NCELLS,
     &        NUMCOL,NUMROW,NUMLAY,
     &        IRWCOL
C
      EXTERNAL OPEN_FILE,CLOSE_FILE,DELETE_FILE
C
      DATA FILES /'budgetitems',
     &            'budgetitem',
     &            'arccmd'/
      DATA IOUT /91,92,93/
      DATA ITEMS /'SEQNUM','ROWCOL','ROW','COLUMN','LAYER_'/
      DATA DECIMLS /4*0, 6/
      DATA TYPES /'I','C',2*'I','F'/
      DATA WIDTHS /4*10,8/
      DATA OUTPUTS /4*10,16/
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          CELL-BY-CELL BUDGET TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.0 24SEPTEMBER1999 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                  GIS OUTPUT MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1----Set local variables
C
      NCELLS=0
      IVIEW=INT(COVPMR(10))
      IRWCL=INT(COVPMR(11))
      MODFLG=INT(COVPMR(1))
      MODFMT=MODFLG-3
      MODPER=INT(COVPMR(13))
      MODSTP=INT(COVPMR(14))
      NPER=INT(COVPMR(17))
C
      IF(NPER.EQ.0) THEN
         MODPER=1
         MODSTP=1
      ENDIF
C
      IF(MODFMT.EQ.1) THEN
         MODTXT='STOTRM'
         WRITE(*,'(
     &          ''      .Processing Cell-By-Cell Storage Terms from '',
     &          ''Block-Centered Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Cell-By-Cell Storage Terms from '',
     &          ''Block-Centered Package '')')
      ELSE IF(MODFMT.EQ.2) THEN
         MODTXT='RIVTRM'
         WRITE(*,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''River Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''River Package '')')
      ELSE IF(MODFMT.EQ.3) THEN
         MODTXT='RCHTRM'
         WRITE(*,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''Recharge Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''Recharge Package '')')
      ELSE IF(MODFMT.EQ.4) THEN
         MODTXT='WELTRM'
         WRITE(*,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''Well Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''Well Package '')')
      ELSE IF(MODFMT.EQ.5) THEN
         MODTXT='DRNTRM'
         WRITE(*,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''Drain Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''Drain Package '')')
      ELSE IF(MODFMT.EQ.6) THEN
         MODTXT='GHBTRM'
         WRITE(*,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''General Head Boundary Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''General Head Boundary Package '')')
      ELSE IF(MODFMT.EQ.7) THEN
         MODTXT='EVTTRM'
         WRITE(*,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''Evapotranspiration Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''Evapotranspiration Package '')')
      ELSE IF(MODFMT.EQ.8) THEN
         MODTXT='STRTRM'
         WRITE(*,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''Streamflow-Routing Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Cell-By-Cell Flow Terms from '',
     &          ''Streamflow-Routing Package '')')
      ELSE IF(MODFMT.EQ.9) THEN
         MODTXT='STRFLO'
         WRITE(*,'(
     &          ''      .Processing Cell-By-Cell Streamflow Terms  '',
     &          ''from Streamflow-Routing Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Cell-By-Cell Streamflow Terms '',
     &          ''from Streamflow-Routing Package '')')
      ELSE IF(MODFMT.EQ.10) THEN
         MODTXT='IBSTRM'
         WRITE(*,'(
     &          ''      .Processing Interbed-Storage Terms from '',
     &          ''Aquifer-System Compaction Package '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Interbed-Storage Terms from '',
     &          ''Aquifer-System Compaction Package '')')
      ELSE
         MODTXT='UNKTRM'
         WRITE(*,'(
     &          ''      .Processing Gridded Terms '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Gridded Terms '')')
      ENDIF
C
C2----Delete and create files for attribute definitions, attribute value, 
C      and coverage arc data and tolerance/dangle values
C
      NSTAT=2
      ITALK=1
      IF(IVIEW.LE.0 .OR. IVIEW.GE.4) 
     &   FILES(2) = BUDFIL(1:INDEX(BUDFIL,' ')-1)
C
C2A----Delete/create files containing Arc/Info item definitions for
C      attribute files
C
      DO 200 N=1,MAXFIL-1
         CALL DELETE_FILE (FILES(N),*9999)
         CALL OPEN_FILE (IOUT(N),FILES(N),NSTAT,ITALK,*9999)
  200 CONTINUE
C
C2B----Calculate rowcol attribute definition
C
      IF(NROW.LT.10) THEN
         LENROW=1
      ELSE IF(NROW.GT.9 .AND. NROW.LT.100) THEN
         LENROW=2
      ELSE IF(NROW.GT.99 .AND. NROW.LT.1000) THEN
         LENROW=3
      ELSE
         LENROW=4
      ENDIF
C
      IF(NCOL.LT.10) THEN
         LENCOL=1
      ELSE IF(NCOL.GT.9 .AND. NCOL.LT.100) THEN
         LENCOL=2
      ELSE IF(NCOL.GT.99 .AND. NCOL.LT.1000) THEN
         LENCOL=3
      ELSE
         LENCOL=4
      ENDIF
C
      IF(NLAY.LT.10) THEN
         LENLAY=1
      ELSE IF(NLAY.GT.9 .AND. NLAY.LT.100) THEN
         LENLAY=2
      ELSE IF(NLAY.GT.99 .AND. NLAY.LT.1000) THEN
         LENLAY=3
      ELSE
         LENLAY=4
      ENDIF
C
      WIDTHS(1)=LENROW+LENCOL
      OUTPUTS(1)=LENROW+LENCOL
      WIDTHS(2)=LENROW+LENCOL
      OUTPUTS(2)=LENROW+LENCOL
      WIDTHS(3)=LENROW
      OUTPUTS(3)=LENROW
      WIDTHS(4)=LENCOL
      OUTPUTS(4)=LENCOL
C
C2C----Write standard attribute definitions
C
      IF(IVIEW.LE.0 .OR. IVIEW.GE.4) THEN
         DO 210 K=1,4
            DECIML=DECIMLS(K)
            ITEM=ITEMS(K)
            OUTPUT=OUTPUTS(K)
            TYPITM=TYPES(K)
            WIDTH=WIDTHS(K)
            WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                      I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
  210    CONTINUE
C
C2Ca----Write attribute definitions IVIEW = all layers
C
        IF(IVIEW.LE.0) THEN
           DECIML=DECIMLS(MAXITM)
           ITEM=ITEMS(MAXITM)
           OUTPUT=OUTPUTS(MAXITM)
           TYPITM=TYPES(MAXITM)
           WIDTH=WIDTHS(MAXITM)
           DO 230 ILAY=1,NLAY
            IF(ILAY.LT.10) THEN
               WRITE(CHRLAY,'(I1)') ILAY
            ELSE IF(ILAY.GT.9 .AND. ILAY.LT.100) THEN
               WRITE(CHRLAY,'(I2)') ILAY
            ELSE IF(ILAY.GT.99 .AND. ILAY.LT.1000) THEN
               WRITE(CHRLAY,'(I3)') ILAY
            ELSE IF(ILAY.GT.999 .AND. ILAY.LT.1000) THEN
               WRITE(CHRLAY,'(I4)') ILAY
            ENDIF
              ITEM='LAYER_'//CHRLAY
              WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                        I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
  230      CONTINUE
C
C2Cb----Write attribute definitions IVIEW = single layer
C
        ELSE IF(IVIEW.EQ.4) THEN
           DECIML=DECIMLS(MAXITM)
           ITEM=ITEMS(MAXITM)
           OUTPUT=OUTPUTS(MAXITM)
           TYPITM=TYPES(MAXITM)
           WIDTH=WIDTHS(MAXITM)
           ILAY=IRWCL
         IF(ILAY.LT.10) THEN
            WRITE(CHRLAY,'(I1)') ILAY
         ELSE IF(ILAY.GT.9 .AND. ILAY.LT.100) THEN
            WRITE(CHRLAY,'(I2)') ILAY
         ELSE IF(ILAY.GT.99 .AND. ILAY.LT.1000) THEN
            WRITE(CHRLAY,'(I3)') ILAY
         ELSE IF(ILAY.GT.999 .AND. ILAY.LT.1000) THEN
            WRITE(CHRLAY,'(I4)') ILAY
         ENDIF
           ITEM='LAYER_'//CHRLAY
           WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                     I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
C
C2Cb----Write attribute definitions IVIEW = ibound value
C
        ELSE IF(IVIEW.EQ.5) THEN
           ITEM='IBOUND'
           DECIML=0
           OUTPUT=10
           TYPITM='I'
           WIDTH=10
           WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                     I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
C
           ITEM=MODTXT
           DECIML=DECIMLS(MAXITM)
           OUTPUT=OUTPUTS(MAXITM)
           TYPITM=TYPES(MAXITM)
           WIDTH=WIDTHS(MAXITM)
           WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                     I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
C
C2Ca----Write attribute definitions IVIEW = uppermost active cell
C
        ELSE IF(IVIEW.EQ.6) THEN
           DECIML=DECIMLS(4)
           ITEM='LAYER'
           OUTPUT=LENLAY
           TYPITM=TYPES(4)
           WIDTH=LENLAY
           WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                     I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
           DECIML=DECIMLS(MAXITM)
           ITEM=ITEMS(MAXITM)
           OUTPUT=OUTPUTS(MAXITM)
           TYPITM=TYPES(MAXITM)
           WIDTH=WIDTHS(MAXITM)
           ITEM=MODTXT
           WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                     I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
         ENDIF
          CALL CLOSE_FILE (FILES(1),*9999)
C
C2D----Write standard attribute definitions
C
      ELSE
         DECIML=0
         ITEM=BUDFIL (1:INDEX(BUDFIL,' ')-1)//'-id'
         OUTPUT=5
         TYPITM='B'
         WIDTH=4
         WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                   I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
         DECIML=DECIMLS(MAXITM)
         ITEM=ITEMS(MAXITM)
         OUTPUT=OUTPUTS(MAXITM)
         TYPITM=TYPES(MAXITM)
         WIDTH=WIDTHS(MAXITM)
         ITEM=MODTXT
         WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                   I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
         CALL CLOSE_FILE (FILES(1),*9999)
      ENDIF
C
C3----Build mapview for single layer (iview=0,4)
C
      IF(IVIEW.LE.0 .OR. IVIEW.EQ.4) THEN
       IF(IVIEW.LE.0) THEN
          WRITE(*,'(''      .Processing cell-by-cell budget terms '',
     &             ''for all layers '',I10)')
     &           NLAY
       ELSE
          WRITE(*,'(''      .Processing cell-by-cell budget terms '',
     &             ''for layer '',I10)')
     &          IRWCL
       ENDIF
C
C3Cb----Grid label attributes
C
         DO 300 IROW=1,NROW
            DO 310 ICOL=1,NCOL
               NCELLS=NCELLS+1
               SEQNUM=(IROW-1)*NCOL+ICOL
               NUMCLL=(IROW-1)*NCOL+ICOL
               ROWCOL=IROW*(10**LENCOL)+ICOL
               WRITE(CHRWCL,'(I10)') ROWCOL
               FINCHR=10
               STRCHR=FINCHR-(LENROW+LENCOL-1)
             IF(IVIEW.LE.0) THEN
                WRITE(IOUT(2),'(
     &                          I10,'','',
     &                          A,'','',
     &                          2(I10,'',''),
     &                          200(F16.3,'',''))')
     &                       SEQNUM,
     &                       CHRWCL (STRCHR:FINCHR),
     &                       IROW,
     &                       ICOL,
     &                       (BUDARY(ICOL,IROW,NUMLAY), NUMLAY=1,NLAY)
              ELSE
                WRITE(IOUT(2),'(
     &                          I10,'','',
     &                          A,'','',
     &                          2(I10,'',''),
     &                          F16.3,'','')')
     &                       SEQNUM,
     &                       CHRWCL (STRCHR:FINCHR),
     &                       IROW,
     &                       ICOL,
     &                       BUDARY(ICOL,IROW,IRWCL)
              ENDIF
  310       CONTINUE
  300    CONTINUE
         WRITE(OUTUNT,'(10X,85(''-''))')
C
C4----Build cross-sectional view (iview=1,2,3)
C
      ELSE IF(IVIEW.GE.1 .AND. IVIEW.LE.3) THEN
       IF(IVIEW.EQ.1) THEN
          WRITE(*,'(''      .Processing cell-by-cell budget terms '',
     &             ''along row '',I10,'' now...'')')
     &          IRWCL
       ELSE IF(IVIEW.EQ.2) THEN
          WRITE(*,'(''      .Processing cell-by-cell budget terms '',
     &             ''along column '',I10,'' now...'')')
     &          IRWCL
       ELSE
          WRITE(*,'(''      .Processing cell-by-cell budget terms '',
     &             ''along particle '',I10,'' now...'')')
     &          IRWCL
       ENDIF
C
C4H----Data header
C
         WRITE(OUTUNT,'(//,1H1,15X,
     &               '' CELLS WITH INVALID VALUES '')')
         WRITE(OUTUNT,'(10X,85(''-''))')
         WRITE(OUTUNT,'(10X,
     &               ''    BUDGET    VALUES          '',
     &               ''       ROW'',
     &               ''    COLUMN'',
     &               ''     LAYER'',
     &               ''    IBOUND'')')
         WRITE(OUTUNT,'(10X,85(''-''))')
C
C4A----Set variables
C
         BADIBD=0
         NUMCLL=0
         NCELLS=0
C
       IF(IVIEW.EQ.1) THEN
          STRCLL=1
          FINCLL=NCOL
       ELSE IF(IVIEW.EQ.2) THEN
          STRCLL=NCOL+1
          FINCLL=STRCLL+NROW-1
       ELSE
          STRCLL=1
          FINCLL=IRWCL
       ENDIF
C
C4B----Loop to build cell arcs from elevation and row/column data
C
         DO 400 N=STRCLL,FINCLL
            NUMCLL=NUMCLL+1
          IF(IVIEW.LE.1) THEN
             IROW=IRWCL
             ICOL=NUMCLL
          ELSE IF(IVIEW.EQ.2) THEN
             IROW=NUMCLL
             ICOL=IRWCL
          ELSE
             ICOL=LRWCL(1,NUMCLL)
             IROW=LRWCL(2,NUMCLL)
          ENDIF
C
C4Ba----Write attributes of each cell
C
            DO 410 ILAY=1,NLAY
               IBDCEL=IBOUND(ICOL,IROW,ILAY)
             IF(IBDCEL.NE.0) THEN
                NCELLS=NCELLS+1
                BUDGET=BUDARY(ICOL,IROW,ILAY)
                WRITE(IOUT(2),'(I10,'','',
     &                          F16.3,'','')')
     &                       NCELLS,
     &                       BUDGET
                WRITE(OUTUNT,'(10X,F20.6,10X,4I10)')
     &                          BUDGET,
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
C
              IF(DELZCB(ILAY).GT.0.0) THEN
                 NUMCON=NUMCON+1
                 WRITE(OUTUNT,'(10X,
     &                   ''CONFINING LAYER CELL          '',
     &                   4I10)')
     &                           IROW,
     &                           ICOL,
     &                           ILAY,
     &                           IBDCEL
                 NCELLS=NCELLS+1
                 BUDGET=0.0
                 WRITE(IOUT(2),'(I10,'','',
     &                           F16.3,'','')')
     &                          NCELLS,
     &                          BUDGET
              ENDIF
             ELSE
                BADIBD=BADIBD+1
                WRITE(OUTUNT,'(10X,
     &                  ''IBOUND = 0                    '',
     &                  4I10)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
             ENDIF
  410       CONTINUE
  400    CONTINUE
         WRITE(OUTUNT,'(10X,85(''-''))')
C
C5----Build mapview for single ibound value (iview=5)
C
      ELSE IF(IVIEW.EQ.5) THEN
          WRITE(*,'(''      .Processing cell-by-cell budget terms '',
     &             ''for ibound value '',I10,'' now...'')')
     &          IRWCL
C
C5A----Grid label attributes
C
         LAYIBD=IRWCL
C
         DO 500 IROW=1,NROW
            DO 510 ICOL=1,NCOL
               BUDIBD=0.0
               DO 520 ILAY=1,NLAY
                  IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
                IF(IBDCEL.EQ.LAYIBD) THEN
                   NCELLS=NCELLS+1
                   BUDIBD=BUDIBD+BUDARY(ICOL,IROW,ILAY)
                ENDIF
  520          CONTINUE
C
               SEQNUM=(IROW-1)*NCOL+ICOL
               NUMCLL=(IROW-1)*NCOL+ICOL
               ROWCOL=IROW*(10**LENCOL)+ICOL
               WRITE(CHRWCL,'(I10)') ROWCOL
               FINCHR=10
               STRCHR=FINCHR-(LENROW+LENCOL-1)
               WRITE(IOUT(2),'(
     &                         I10,'','',
     &                         A,'','',
     &                         3(I10,'',''),
     &                         F16.3,'','')')
     &                      SEQNUM,
     &                      CHRWCL (STRCHR:FINCHR),
     &                      IROW,
     &                      ICOL,
     &                      LAYIBD,
     &                      BUDIBD
  510       CONTINUE
  500    CONTINUE
         WRITE(OUTUNT,'(10X,85(''-''))')
C
C6----Build mapview for uppermost active cell (iview=6)
C
      ELSE
          WRITE(*,'(''      .Processing cell-by-cell budget terms '',
     &             ''uppermost active cell...'')')
C
C6A----Grid cell attributes
C
         DO 600 IROW=1,NROW
            DO 610 ICOL=1,NCOL
               NCELLS=NCELLS+1
               DO 620 ILAY=1,NLAY
                  IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
                IF(IBDCEL.NE.0) THEN
                   NCELLS=NCELLS+1
                   SEQNUM=(IROW-1)*NCOL+ICOL
                   NUMCLL=(IROW-1)*NCOL+ICOL
                   ROWCOL=IROW*(10**LENCOL)+ICOL
                   WRITE(CHRWCL,'(I10)') ROWCOL
                   FINCHR=10
                   STRCHR=FINCHR-(LENROW+LENCOL-1)
                   WRITE(IOUT(2),'(
     &                             I10,'','',
     &                             A,'','',
     &                             3(I10,'',''),
     &                             F16.3,'','')')
     &                          SEQNUM,
     &                          CHRWCL (STRCHR:FINCHR),
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          BUDARY(ICOL,IROW,ILAY)
                   GO TO 610
                ENDIF
  620          CONTINUE
  610       CONTINUE
  600    CONTINUE
         WRITE(OUTUNT,'(10X,85(''-''))')
      ENDIF
C
C7----Close attribute files
C
      CALL CLOSE_FILE (FILES(2),*9999)
C
C8----Messages
C
C
C8A----Message to user
C
      WRITE(*,'(''      .Cell-by-cell budget term file '',A,
     &          '' consists of '')')
     &        BUDFIL (1:INDEX(BUDFIL,' ')-1)
      WRITE(*,'(''      .'',I10,'' cells'')') NCELLS
      WRITE(*,'(''      .'',I10,'' cells were '',
     &          ''ommited (ibound = 0)'')') BADIBD
      WRITE(*,'(''      .'',I10,'' cells were '',
     &          ''in confining layer'')') NUMCON
C
C8B----Message to operational output file
C
      WRITE(OUTUNT,'(''      .Cell-by-cell budget term file '',
     &               A,
     &               '' consists of '')')
     &            BUDFIL (1:INDEX(BUDFIL,' ')-1)
      WRITE(OUTUNT,'(''      .'',I10,'' cells'')') NCELLS
      WRITE(OUTUNT,'(''      .'',I10,'' cells were '',
     &          ''ommited (ibound = 0)'')') BADIBD
      WRITE(OUTUNT,'(''      .'',I10,'' cells were '',
     &          ''in confining layer'')') NUMCON
C
C9----Arc Operations to coverage and attribute files
C
      IF(IVIEW.NE.3) THEN
         CALL DELETE_FILE (FILES(MAXFIL),*9999)
         CALL OPEN_FILE (IOUT(MAXFIL),FILES(MAXFIL),NSTAT,ITALK,*9999)
      ELSE
         FILES(1)='arccmd.old'
         CALL DELETE_FILE (FILES(1),*9999)
         FILES(MAXFIL)='arccmd'
         CALL OPEN_FILE (IOUT(MAXFIL),FILES(1),NSTAT,ITALK,*9999)
         DO 900 N=1,100
            READ(IOUT(MAXFIL),'(A)',END=902) ARCCMD
            WRITE(IOUT(1),'(A)') ARCCMD
  900    CONTINUE
  902    CALL CLOSE_FILE (FILES(MAXFIL),*9999)
         CALL DELETE_FILE (FILES(MAXFIL),*9999)
         CALL OPEN_FILE (IOUT(MAXFIL),FILES(MAXFIL),NSTAT,ITALK,*9999)
         CALL CLOSE_FILE (FILES(1),*9999)
         CALL OPEN_FILE (IOUT(1),FILES(1),NSTAT,ITALK,*9999)
         DO 910 N=1,100
            READ(IOUT(1),'(A)',END=912) ARCCMD
            WRITE(IOUT(MAXFIL),'(A)') ARCCMD
  910    CONTINUE
  912    CALL CLOSE_FILE (FILES(1),*9999)
         CALL DELETE_FILE (FILES(1),*9999)
      ENDIF
C
C9A----Create attribute files and "add from" attributes
C
      IF(IVIEW.LE.0 .OR. IVIEW.GE.4) THEN
         ARCCMD='tables'
         TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1)
         ADDFIL=FILES(2) (1:INDEX(FILES(2),' ')-1)
         WRITE(IOUT(MAXFIL),'(A,2(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1)
C
C9B---Join attribute files (if needed)
C
      ELSE
         ARCCMD='tables'
         TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1)
         ADDFIL=FILES(2) (1:INDEX(FILES(2),' ')-1)
         ACTION='kill'
         WRITE(IOUT(MAXFIL),'(A,3(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1),
     &                             ACTION (1:INDEX(ACTION,' ')-1)
         ARCCMD='joinitem'
         LABFIL=BUDFIL (1:INDEX(BUDFIL,' ')-1)//'.pat'
         INPFIL=FILES(2) (1:INDEX(FILES(2),' ')-1)
         RELITM=BUDFIL (1:INDEX(BUDFIL,' ')-1)//'-id'
         STRITM='thick'
         JONTYP='linear'
         WRITE(IOUT(MAXFIL),'(A,6(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             LABFIL (1:INDEX(LABFIL,' ')-1),
     &                             INPFIL (1:INDEX(INPFIL,' ')-1),
     &                             LABFIL (1:INDEX(LABFIL,' ')-1),
     &                             RELITM (1:INDEX(RELITM,' ')-1),
     &                             STRITM (1:INDEX(STRITM,' ')-1),
     &                             JONTYP (1:INDEX(JONTYP,' ')-1)
      ENDIF
      CALL CLOSE_FILE (FILES(MAXFIL),*9999)
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9999 WRITE(*,
     &    '(//,'' Abnormal termination of Pthbud_Subroutine'')')
      WRITE(OUTUNT,
     &    '(//,'' Abnormal termination of Pthbud_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
