C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************              Omit Utility version 6.0              *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.0 08JULY1996
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     A set of subroutines that replace the utility modules of Modflowarc
C      (Orzol and McGrath, 1991) or Modflow (McDonald and Harbaugh, 1988).
C      These subroutines read data, but do not load the values into
C      the model.
C
C    Subroutines:
C
C     U1DRELOMIT    : ROUTINE TO INPUT 1-D REAL DATA MATRICES
C     U2DINTOMIT    : ROUTINE TO INPUT 2-D INTEGER DATA MATRICES
C     U2DRELOMIT    : ROUTINE TO INPUT 2-D REAL DATA MATRICES
C     U3DRELOMIT    : ROUTINE TO INPUT 3-D REAL DATA MATRICES
C
C    Secondary Subroutines:
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
      SUBROUTINE U1DRELOMIT (A,ANAME,JJ,IN,IOUT,
     E                       *)
C
C
C-----VERSION 6.0 08JULY1996 UD1RELOMIT
C               MODIFIED BY LEONARD L. ORZOL
C
C     ******************************************************************
C     ROUTINE TO INPUT 1-D REAL DATA MATRICES
C       A IS ARRAY TO INPUT
C       ANAME IS 24 CHARACTER DESCRIPTION OF A
C       JJ IS NO. OF ELEMENTS
C       IN IS INPUT UNIT
C       IOUT IS OUTPUT UNIT
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*4 ANAME
      CHARACTER*80 CNTRL,FNAME
      CHARACTER*20 FMTIN
      DIMENSION A(JJ),ANAME(6)
      DATA NUNOPN/99/
C     ------------------------------------------------------------------
C
C1------READ ARRAY CONTROL RECORD.
C
      READ(IN,'(A)', 
     E        ERR=9990,
     E        END=9991)
     &                  CNTRL
C
C2------LOOK FOR ALPHABETIC WORD THAT INDICATES THAT THE RECORD IS FREE
C2------FORMAT.  SET A FLAG SPECIFYING IF FREE FORMAT OR FIXED FORMAT.
      ICLOSE=0
      IFREE=1
      ICOL=1
      CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,IOUT,IN)
      IF(CNTRL(ISTART:ISTOP).EQ.'CONSTANT') THEN
         LOCAT=0
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'INTERNAL') THEN
         LOCAT=IN
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'EXTERNAL') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,LOCAT,R,IOUT,IN)
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'OPEN/CLOSE') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,0,N,R,IOUT,IN)
         FNAME=CNTRL(ISTART:ISTOP)
         LOCAT=NUNOPN
         WRITE(IOUT,201) ANAME,LOCAT,FMTIN
         GO TO 502
      ELSE
C
C2A-----DID NOT FIND A RECOGNIZED WORD, SO NOT USING FREE FORMAT.
C2A-----READ THE CONTROL RECORD THE ORIGINAL WAY.
         IFREE=0
         READ (CNTRL,
     F               '(I10,F10.0,A20,I10)',
     E               ERR=9990,
     E               END=9991)
     &                                      LOCAT,CNSTNT,FMTIN,IPRN
      ENDIF
C
C3------FOR FREE FORMAT CONTROL RECORD, READ REMAINING FIELDS.
      IF(IFREE.NE.0) THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,3,N,CNSTNT,IOUT,IN)
       IF(LOCAT.GT.0) THEN
          CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,IOUT,IN)
          FMTIN=CNTRL(ISTART:ISTOP)
          CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,IPRN,R,IOUT,IN)
       ENDIF
      ENDIF
C
C2------IF LOCAT NOT EQUAL TO IN THEN RETURN
C
      IF(LOCAT.GT.0 .AND. LOCAT.NE.IN) THEN
         WRITE(IOUT,201) ANAME,LOCAT,FMTIN
  201    FORMAT(1H0,///30X,6A4,' WILL NOT BE READ ON UNIT',I3,
     &         ' USING FORMAT: ',A20/10X,79('-')/)
C
C3------IF LOCAT=0 THEN RETURN
C
      ELSE IF(LOCAT.EQ.0) THEN
         WRITE(IOUT,301) ANAME
  301    FORMAT(1H0,///30X,6A4,' WILL NOT BE READ ',
     &          /10X,79('-')/)
C
C4A-----IF LOCAT EQUALS IN THEN READ RECORDS.
C
      ELSE
       IF(FMTIN.EQ.'(FREE)') THEN
          READ(LOCAT,*) (A(J),J=1,JJ)
       ELSE
          WRITE(IOUT,201) ANAME,LOCAT,FMTIN
          READ (LOCAT,FMTIN,
     E                ERR=9992,
     E                END=9993)
     &                          (A(J), J=1,JJ)
       ENDIF
C
C4B-----CLOSE IF KEYWORD = OPEN/CLOSE
C
       IF(ICLOSE.NE.0) CLOSE(UNIT=LOCAT)
      ENDIF
       GOTO 502
C
CE------ERRORS
C
 9990 WRITE(*,'(//,'' Unable to read from control line '',
     &          '' for item '',6A,/,
     &          ''  Reformat control line'')')
     &        ANAME
      GOTO 9999
 9991 WRITE(*,'(//,'' End-of-file reached for item '',6A,/,
     &          ''; Missing input package control line'')')
     &        ANAME
      GOTO 9999
 9992 WRITE(*,'(//,'' Unable to read for item '',6A,''; Bad format'')')
     &        ANAME
      GOTO 9999
 9993 WRITE(*,'(//,'' End-of-file reached for item '',6A)')  
     &        ANAME
 9999 WRITE(*,'(//,'' Abnormal Termination of U1drelomit_Subroutine'')')
      RETURN 1
C
C5------RETURN
C
 502  RETURN
      END
C
      SUBROUTINE U2DINTOMIT (IA,ANAME,II,JJ,K,IN,IOUT,
     E                       *)
C
C
C-----VERSION 4.0 30MARCH1993 U2DINTOMIT
C               MODIFIED BY LEONARD L. ORZOL
C
C     ******************************************************************
C     ROUTINE TO INPUT 2-D INTEGER DATA MATRICES
C       IA IS ARRAY TO INPUT
C       ANAME IS 24 CHARACTER DESCRIPTION OF IA
C       II IS NO. OF ROWS
C       JJ IS NO. OF COLS
C       K IS LAYER NO. (USED WITH NAME TO TITLE PRINTOUT UNLESS K IS 0)
C       IN IS INPUT UNIT
C       IOUT IS OUTPUT UNIT
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*4 ANAME
      CHARACTER*80 CNTRL,FNAME
      CHARACTER*20 FMTIN
      DIMENSION IA(JJ,II),ANAME(6)
      DATA NUNOPN/99/
C     ------------------------------------------------------------------
C
C1------READ ARRAY CONTROL RECORD.
C
      READ(IN,'(A)', 
     E        ERR=9990,
     E        END=9991)
     &                  CNTRL
C
C2------LOOK FOR ALPHABETIC WORD THAT INDICATES THAT THE RECORD IS FREE
C2------FORMAT.  SET A FLAG SPECIFYING IF FREE FORMAT OR FIXED FORMAT.
      ICLOSE=0
      IFREE=1
      ICOL=1
      CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,IOUT,IN)
      IF(CNTRL(ISTART:ISTOP).EQ.'CONSTANT') THEN
         LOCAT=0
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'INTERNAL') THEN
         LOCAT=IN
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'EXTERNAL') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,LOCAT,R,IOUT,IN)
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'OPEN/CLOSE') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,0,N,R,IOUT,IN)
         FNAME=CNTRL(ISTART:ISTOP)
         LOCAT=NUNOPN
         WRITE(IOUT,201) ANAME,LOCAT,FMTIN
         GO TO 502
      ELSE
C
C2A-----DID NOT FIND A RECOGNIZED WORD, SO NOT USING FREE FORMAT.
C2A-----READ THE CONTROL RECORD THE ORIGINAL WAY.
         IFREE=0
         READ (CNTRL,
     F               '(I10,F10.0,A20,I10)',
     E               ERR=9990,
     E               END=9991)
     &                                      LOCAT,CNSTNT,FMTIN,IPRN
      ENDIF
C
C3------FOR FREE FORMAT CONTROL RECORD, READ REMAINING FIELDS.
      IF(IFREE.NE.0) THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,3,N,CNSTNT,IOUT,IN)
         IF(LOCAT.GT.0) THEN
            CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,IOUT,IN)
            FMTIN=CNTRL(ISTART:ISTOP)
            CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,IPRN,R,IOUT,IN)
         ENDIF
      ENDIF
C
C2------IF LOCAT NOT EQUAL TO IN THEN RETURN
C
      IF(LOCAT.GT.0 .AND. LOCAT.NE.IN) THEN
         WRITE(IOUT,201) ANAME,LOCAT,FMTIN
  201    FORMAT(1H0,///30X,6A4,' WILL NOT BE READ ON UNIT',I3,
     &         ' USING FORMAT: ',A20/10X,79('-')/)
C
C3------IF LOCAT=0 THEN RETURN
C
      ELSE IF(LOCAT.EQ.0) THEN
         WRITE(IOUT,301) ANAME
  301    FORMAT(1H0,///30X,6A4,' WILL NOT BE READ ',
     &          /10X,79('-')/)
C
C4A-----IF LOCAT EQUALS IN THEN READ RECORDS.
C
      ELSE
         DO 400 I=1,II
          IF(FMTIN.EQ.'(FREE)') THEN
             READ(LOCAT,*) (IA(J,I),J=1,JJ)
          ELSE
             WRITE(IOUT,201) ANAME,LOCAT,FMTIN
             READ (LOCAT,FMTIN,
     E                ERR=9992,
     E                END=9993)
     &                          (IA(J,I), J=1,JJ)
          ENDIF
 400     CONTINUE
C
C4B-----CLOSE IF KEYWORD = OPEN/CLOSE
C
       IF(ICLOSE.NE.0) CLOSE(UNIT=LOCAT)
      ENDIF
       GOTO 502
C
CE------ERRORS
C
 9990 WRITE(*,'(//,'' Unable to read from control line '',/,
     &          '' for layer '',I10,
     &          '' and item '',6A,''; Reformat control line'')')
     &        K,
     &        ANAME
      GOTO 9999
 9991 WRITE(*,'(//,'' End-of-file reached for layer '',I10,
     &          '' and item '',6A
     &          ''; Missing input package control line'')')
     &        K,
     &        ANAME
      GOTO 9999
 9992 WRITE(*,'(//,'' Unable to read for layer '',I10,
     &          '' and item '',6A,''; Bad format'')')
     &        K,
     &        ANAME
      GOTO 9999
 9993 WRITE(*,'(//,'' End-of-file reached for layer '',I10,
     &          '' and item '',6A)')
     &        K,
     &        ANAME
 9999 WRITE(*,'(//,'' Abnormal Termination of U2dintomit_Subroutine'')')
      RETURN 1
C
C5-----RETURN
C
 502  RETURN
      END
C
      SUBROUTINE U2DRELOMIT (A,ANAME,II,JJ,K,IN,IOUT,
     E                       *)
C
C
C-----VERSION 4.0 30MARCH1993 U2DRELOMIT
C               MODIFIED BY LEONARD L. ORZOL
C
C     ******************************************************************
C     ROUTINE TO INPUT 2-D REAL DATA MATRICES
C       A IS ARRAY TO INPUT
C       ANAME IS 24 CHARACTER DESCRIPTION OF A
C       II IS NO. OF ROWS
C       JJ IS NO. OF COLS
C       K IS LAYER NO. (USED WITH NAME TO TITLE PRINTOUT UNLESS K IS 0)
C       IN IS INPUT UNIT
C       IOUT IS OUTPUT UNIT
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*4 ANAME
      CHARACTER*80 CNTRL,FNAME
      CHARACTER*20 FMTIN
      DIMENSION A(JJ,II),ANAME(6)
      DATA NUNOPN/99/
C     ------------------------------------------------------------------
C
C1------READ ARRAY CONTROL RECORD.
C
      READ(IN,'(A)',
     E        ERR=9990,
     E        END=9991)
     &                  CNTRL
C
C2------LOOK FOR ALPHABETIC WORD THAT INDICATES THAT THE RECORD IS FREE
C2------FORMAT.  SET A FLAG SPECIFYING IF FREE FORMAT OR FIXED FORMAT.
      ICLOSE=0
      IFREE=1
      ICOL=1
      CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,IOUT,IN)
      IF(CNTRL(ISTART:ISTOP).EQ.'CONSTANT') THEN
         LOCAT=0
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'INTERNAL') THEN
         LOCAT=IN
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'EXTERNAL') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,LOCAT,R,IOUT,IN)
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'OPEN/CLOSE') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,0,N,R,IOUT,IN)
         FNAME=CNTRL(ISTART:ISTOP)
         LOCAT=NUNOPN
         WRITE(IOUT,201) ANAME,LOCAT,FMTIN
         GO TO 502
      ELSE
C
C2A-----DID NOT FIND A RECOGNIZED WORD, SO NOT USING FREE FORMAT.
C2A-----READ THE CONTROL RECORD THE ORIGINAL WAY.
         IFREE=0
         READ (CNTRL,
     F               '(I10,F10.0,A20,I10)',
     E               ERR=9990,
     E               END=9991)
     &                                      LOCAT,CNSTNT,FMTIN,IPRN
      ENDIF
C
C3------FOR FREE FORMAT CONTROL RECORD, READ REMAINING FIELDS.
      IF(IFREE.NE.0) THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,3,N,CNSTNT,IOUT,IN)
       IF(LOCAT.GT.0) THEN
          CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,IOUT,IN)
          FMTIN=CNTRL(ISTART:ISTOP)
          CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,IPRN,R,IOUT,IN)
       ENDIF
      ENDIF
C
C2------IF LOCAT NOT EQUAL TO IN THEN RETURN
C
      IF(LOCAT.GT.0 .AND. LOCAT.NE.IN) THEN
         WRITE(IOUT,201) ANAME,LOCAT,FMTIN
  201    FORMAT(1H0,///30X,6A4,' WILL NOT BE READ ON UNIT',I3,
     &         ' USING FORMAT: ',A20/10X,79('-')/)
C
C3------IF LOCAT=0 THEN RETURN
C
      ELSE IF(LOCAT.EQ.0) THEN
         WRITE(IOUT,301) ANAME
  301    FORMAT(1H0,///30X,6A4,' WILL NOT BE READ ',
     &          /10X,79('-')/)
C
C4A-----IF LOCAT EQUALS IN THEN READ RECORDS.
C
      ELSE
         WRITE(IOUT,203) ANAME,LOCAT,FMTIN
 203     FORMAT(1H0,///30X,6A4,' WILL BE READ ON UNIT',I3,
     &         ' USING FORMAT: ',A20/10X,79('-')/)
         DO 400 I=1,II
          IF(FMTIN.EQ.'(FREE)') THEN
             READ(LOCAT,*) (A(J,I),J=1,JJ)
          ELSE
             READ (LOCAT,FMTIN,
     E                ERR=9992,
     E                END=9993)
     &                          (A(J,I), J=1,JJ)
          ENDIF
 400     CONTINUE
      ENDIF
C
C4B-----CLOSE IF KEYWORD = OPEN/CLOSE
C
       IF(ICLOSE.NE.0) CLOSE(UNIT=LOCAT)
        GOTO 502
C
CE------ERRORS
C
 9990 WRITE(*,'(//,'' Unable to read from control line '',/,
     &          '' for layer '',I10,
     &          '' and item '',6A,''; Reformat control line'')')
     &        K,
     &        ANAME
      GOTO 9999
 9991 WRITE(*,'(//,'' End-of-file reached for layer '',I10,
     &          '' and item '',6A
     &          ''; Missing input package control line'')')
     &        K,
     &        ANAME
      GOTO 9999
 9992 WRITE(*,'(//,'' Unable to read for layer '',I10,
     &          '' and item '',6A,''; Bad format'')')
     &        K,
     &        ANAME
      GOTO 9999
 9993 WRITE(*,'(//,'' End-of-file reached for layer '',I10,
     &          '' and item '',6A)')
     &        K,
     &        ANAME
 9999 WRITE(*,'(//,'' Abnormal Termination of U2drelomit_Subroutine'')')
      RETURN 1
C
C5-----RETURN
C
 502  RETURN
      END
C
      SUBROUTINE U3DRELOMIT (A,ANAME,II,JJ,KK,IN,IOUT,
     E                       *)
C
C
C-----VERSION 4.0 30MARCH1993 U3DRELOMIT
C               BY LEONARD L. ORZOL
C
C     ******************************************************************
C     ROUTINE TO INPUT 3-D REAL DATA MATRICES
C       A IS ARRAY TO INPUT
C       ANAME IS 24 CHARACTER DESCRIPTION OF A
C       II IS NO. OF ROWS
C       JJ IS NO. OF COLS
C       KK IS LAYER NO.
C       IN IS INPUT UNIT
C       IOUT IS OUTPUT UNIT
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*4 ANAME
      CHARACTER*20 FMTIN
      DIMENSION A(JJ,II,KK),ANAME(6)
C     ------------------------------------------------------------------
C
C1------READ ARRAY CONTROL RECORD.
C
      READ(IN,'(I10,F10.0,A20,I10)',
     E          ERR=9990,
     E          END=9991)
     &                                      LOCAT,CNSTNT,FMTIN,IPRN
C
C2------IF LOCAT NOT EQUAL IN THEN RETURN
C
      IF(LOCAT.NE.IN .AND. LOCAT.GT.0) THEN
         WRITE(IOUT,201) ANAME,KK,LOCAT
  201    FORMAT(1H0,///30X,6A4,' FOR ',I3,' LAYERS',
     &           ' WILL NOT BE READ ON UNIT',I3,/30X,96('-'))
C
C3------IF LOCAT=0 THEN RETURN
C
      ELSE IF(LOCAT.EQ.0) THEN
         WRITE(IOUT,301) ANAME,KK
  301    FORMAT(1H0,52X,6A4,' WILL BE OMITTED FOR ',I3,' LAYERS',
     &          /30X,83('-'))
C
C4------IF LOCAT EQUALS IN THEN READ RECORDS.
C
      ELSE
         WRITE(IOUT,201) ANAME,KK,LOCAT
         DO 400 I=1,II
            DO 410 J=1,JJ
               READ(LOCAT,FMTIN,
     E                    ERR=9992,
     E                    END=9993)
     &                              (A(J,I,K), K=1,KK)
  410       CONTINUE
  400   CONTINUE
      ENDIF
       GOTO 502
C
CE------ERRORS
C
 9990 WRITE(*,'(//,'' Unable to read from control line '',/,
     &          '' for layer '',I10,
     &          '' and item '',6A,''; Reformat control line'')')
     &        K,
     &        ANAME
      GOTO 9999
 9991 WRITE(*,'(//,'' End-of-file reached for layer '',I10,
     &          '' and item '',6A
     &          ''; Missing input package control line'')')
     &        K,
     &        ANAME
      GOTO 9999
 9992 WRITE(*,'(//,'' Unable to read for layer '',I10,
     &          '' and item '',6A,''; Bad format'')')
     &        K,
     &        ANAME
      GOTO 9999
 9993 WRITE(*,'(//,'' End-of-file reached for layer '',I10,
     &          '' and item '',6A)')
     &        K,
     &        ANAME
 9999 WRITE(*,'(//,'' Abnormal Termination of U3drelomit_Subroutine'')')
      RETURN 1
C
C5-----RETURN
C
 502  RETURN
      END
